package ca.canadapost.cpcdp.postoffice;

import ca.canadapost.cpcdp.postoffice.stub.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.postoffice.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetNearestPostOffice Canada Post service.
 * <p>
 * The GetNearestPostOffice service returns a list of Post Offices
 * near a given location including information such as address and phone number.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetNearestPostOfficeClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_PostOffice_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");

            PostOfficeService service = new PostOfficeService();
            PostOfficePortType port = service.getPostOfficePort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/postoffice");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetNearestPostOfficeRequest.SearchData searchData = new GetNearestPostOfficeRequest.SearchData();
            searchData.setPostalCode("K2B8J6");

            GetNearestPostOfficeRequest request = new GetNearestPostOfficeRequest();
            request.setLocale("FR");
            request.setMaximum(3);
            request.setSearchData(searchData);

            //execute request
            GetNearestPostOfficeResponse response = port.getNearestPostOffice(request);

            //retrieve values from response
            if(response.getMessages() == null){
                if(response.getPostOfficeList() != null){
                    for(GetNearestPostOfficeResponse.PostOfficeList.PostOffice postOffice : response.getPostOfficeList().getPostOffice()){
                        System.out.println("Office Name: " + postOffice.getName());
                        System.out.println("Office Id: " + postOffice.getOfficeId());
                        System.out.println("Address: " + postOffice.getAddress().getOfficeAddress());
                        System.out.println();
                    }
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
