package ca.canadapost.cpcdp.postoffice;

import ca.canadapost.cpcdp.postoffice.stub.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.postoffice.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetPostOfficeDetail Canada Post service.
 * <p>
 * The GetPostOfficeDetail service returns Post Office details such as the daily
 * hours of operation for a given Post Office as determined by the first call.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetPostOfficeDetailClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_PostOffice_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");

            PostOfficeService service = new PostOfficeService();
            PostOfficePortType port = service.getPostOfficePort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/postoffice");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetPostOfficeDetailRequest request = new GetPostOfficeDetailRequest();
            request.setLocale("FR");
            request.setOfficeId("102978");

            //execute request
            GetPostOfficeDetailResponse response = port.getPostOfficeDetail(request);

            //retrieve values from response
            if(response.getMessages() == null){
                System.out.println("Office Name: " + response.getPostOfficeDetail().getName());
                System.out.println("Address: " + response.getPostOfficeDetail().getAddress().getOfficeAddress());
                System.out.println();
                for(GetPostOfficeDetailResponse.PostOfficeDetail.HoursList hours : response.getPostOfficeDetail().getHoursList()){
                    System.out.println("Weekday: " + hours.getDay());
                    for(String time : hours.getTime()){
                        System.out.println("Time: " + time);
                    }
                    System.out.println();
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
